﻿using System;
using System.Diagnostics.Contracts;

namespace Demo.CodeContracts
{
    [ContractClass(typeof(LinkedListContract))]
    public interface ILinkedList
    {
        int Count { get; }
        void InsertAt(int index, object value);
        object ValueAt(int index);
    }

    [ContractClassFor(typeof(ILinkedList))]
    public abstract class LinkedListContract : ILinkedList
    {
        int ILinkedList.Count
        {
            get
            {
                Contract.Ensures(Contract.Result<int>() >= 0);
                return 0;
            }
        }

        void ILinkedList.InsertAt(int index, object value)
        {
            Contract.Requires<ArgumentOutOfRangeException>(index >= 0);
            Contract.Requires<ArgumentOutOfRangeException>(index <= ((ILinkedList)this).Count);
            Contract.Requires<ArgumentNullException>(value != null);
        }

        object ILinkedList.ValueAt(int index)
        {
            Contract.Requires<ArgumentOutOfRangeException>(index >= 0);
            Contract.Requires<ArgumentOutOfRangeException>(index < ((ILinkedList)this).Count);
            Contract.Ensures(Contract.Result<object>() != null);
            return null;
        }
    }
}