﻿using System;
using System.Collections.Generic;
using System.Diagnostics.Contracts;
using System.Threading.Tasks;

namespace Demo.CodeContracts
{
    public class LinkedList : ILinkedList
    {
        [ContractPublicPropertyName("Count")]
        private int _count;

        public LinkedList()
        {
            FirstNode = new Node(new object(), null);  // Dummy node
            _count = 0;
        }

        public Node FirstNode { get; private set; }

        public int Count
        {
            [Pure]
            get { return _count; }
        }

        public void InsertAt(int index, object value)
        {
            Node node = FirstNode;
            for (int i = 0; i < index; i++)
            {
                Contract.Assume(node != null);
                node = node.Next;
            }

            Contract.Assume(node != null);
            var newNode = new Node(value, node.Next);
            node.Next = newNode;
            ++_count;
        }

        [Pure]
        public object ValueAt(int index)
        {
            Node node = FirstNode.Next;
            for (int i = 0; i < index; i++)
            {
                Contract.Assume(node != null);
                node = node.Next;
            }
            Contract.Assume(node != null);
            return node.Value;
        }

        [Pure]
        public async Task<Node> NodeOfAsync(object value)
        {
            Contract.Requires<ArgumentNullException>(value != null);
            Contract.Ensures(Contract.Result<Task<Node>>() != null);
            // ReSharper disable PossibleNullReferenceException
            Contract.Ensures(Contract.Result<Task<Node>>().Result != null);
            // ReSharper restore PossibleNullReferenceException

            Node node = await Task.Run(() => InternalNodeOf(value));
            return node;
        }

        [Pure]
        private Node InternalNodeOf(object value)
        {
            Contract.Requires(value != null);
            Contract.Ensures(Contract.Result<Node>() != null);

            Node node = FirstNode.Next;
            for (int i = 0; i < Count; i++)
            {
                Contract.Assume(node != null);
                if (node.Value.Equals(value))
                    return node;
                node = node.Next;
            }
            throw new KeyNotFoundException();
        }

        [ContractInvariantMethod]
        private void InvariantMethod()
        {
            Contract.Invariant(_count >= 0);
            Contract.Invariant(FirstNode != null);
        }
    }
}